;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x39e1713641e9865b618df64a8c832610 . "./code.cg2js/cg-common-control.fasl")
(#xfef07b9e3c4ee42d4232d627b1598abe . "./code.cg2js/cg-clipboard-stack.fasl")
(#x75c0de5c22ebb22b9ebef46fa3716091 . "./code.cg2js/cg-yes-no-string.fasl")
(#x93168188fddadf2a6b4dcdde7e3b83e5 . "./code.cg2js/cg-clipboard.fasl")
(#x4f9cb7d5c5efa89ee125650daac961cb . "./code.cg2js/cg-dde-utils.fasl")
(#x5be616b0bb7b8cfcc6f18b6218b586e1 . "./code.cg2js/cg-editable-text.fasl")
(#xdebabf85c72f91110c855d84d6d0f230 . "./code.cg2js/cg-player.fasl")
(#x297d222ab7030aa950508268e5d8e9df . "./code.cg2js/cg-text-or-combo.fasl")
(#x8f368f8e60c1bc30bad22f8b21ba9a0f . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xed00f55c11cfca3e49474911ce3e944e . "./code.cg2js/cg-graphics-context.fasl")
(#x6a04fcf79fa95c4e660bcea21fee1856 . "./code.cg2js/cg-choose-printer.fasl")
(#xf38a6d3c3be57111519502f0a3b8ba96 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xa1953392e98b076bf585e09dea51059b . "./code.cg2js/cg-edit-in-place.fasl")
(#x345287b7e277d11e9f4909bbba9a8a99 . "./code.cg2js/cg-split-bar.fasl")
(#x98de7382f5341784ad0ef0fcb70a25de . "./code.cg2js/cg-timer.fasl")
(#x25f025df62a03e992070a442ab6a8753 . "./code.cg2js/cg-progress-indicator.fasl")
(#xe8cb58ef35de1a7f52bd918fcee62e28 . "./code.cg2js/cg-object-editor.fasl")
(#x12a2ffda5cdda82c0f76a4ce36a90aae . "./code.cg2js/cg-fill-texture.fasl")
(#x21225bf64a2b17f0cf9759ef17bb93f4 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x38b8410ef064086f19b8a4376514e5ff . "./code.cg2js/cg-os-window.fasl")
(#x5f372a9093ae951f9ec60cd8cf434c3e . "./code.cg2js/cg-color-dialog.fasl")
(#xb88975bb59d03aacdc1c2448df4fe44c . "./code.cg2js/cg-combo-box.fasl")
(#xaaade257f55e11ff419174e2e01069a8 . "./code.cg2js/cg-directory-dialog.fasl")
(#xa27dcde3e7e1771fdacc3d4bb1c151ae . "./code.cg2js/cg-tooltip.fasl")
(#x5045f8e1b6a418dbd09ec0aa75fb773a . "./code.cg2js/cg-lettered-menu.fasl")
(#x5acb626767f55e9d10a2662b3a2e526b . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x9158b94c9ff9187d7d057cea506a262e . "./code.cg2js/cg-pixmap.fasl")
(#x7db12e772c906a6be91c5e943a40f6da . "./code.cg2js/cg.fasl")
(#xa079e60c26f7ea01ad39f36c2045b89a . "./code.cg2js/cg-class-slot-grid.fasl")
(#x96f5234a62692267d69c537216b8e117 . "./code.cg2js/cg-template-string.fasl")
(#x76707137ae85153b1177c9818bb2e1b5 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xf955daf84c625c37fcd5d601bc39c2d7 . "./code.cg2js/cg-check-box.fasl")
(#x1b1af17a400c0904fd296b8f94e0f43a . "./code.cg2js/cg-toggling-widget.fasl")
(#xfb49fda0ceaf53053f66508595f0e1c0 . "./code.cg2js/cg-radio-button.fasl")
(#x7786927d6531013dd2e797c7eb7bd8b3 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x71cc41193b2d0fbf0755c68d413d1aeb . "./code.cg2js/cg-scaling-stream.fasl")
(#xe9bab792348a555391eb8a836065844d . "./code.cg2js/cg-multi-picture-button.fasl")
(#x1f44130db29a2066a5fde43f0e89b6cb . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xd41e409c104722c039ab41db8bbad53f . "./code.cg2js/cg-grid-widget.fasl")
(#xb698936b7a0690c2f3514cab491ed0c7 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xa6bdcf08335c937db2536334d61b3dd2 . "./code.cg2js/cg-item-list.fasl")
(#x514bf00f1582560e1c641e3d5c6295fd . "./code.cg2js/cg-paren-matching.fasl")
(#x30aa22f5b1b4847ea6d5eba8cb7d8d9a . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x811d30dfc55d3a13cbb5a12077d3c8cf . "./code.cg2js/cg-html-dialog.fasl")
(#xc9157a2472278caefe331e0b4c2e417b . "./code.cg2js/cg-pixmap-widget.fasl")
(#xa5c85161158ce86d4ef9e0001d861811 . "./code.cg2js/cg-project-window.fasl")
(#xb273749f7b948e6a477ce72ea67ef5fc . "./code.cg2js/cg-toolbar.fasl")
(#x887dbb624cb38279f0fc005db2973b74 . "./code.cg2js/cg-caret.fasl")
(#x76eba243a0120a84acd3eabe22e3d126 . "./code.cg2js/cg-scroll-bar.fasl")
(#xd8471b074f823822bea90c7e4b383c53 . "./code.cg2js/cg-web-browser.fasl")
(#x9e31a2f65fbec6d24e2457cab9915f19 . "./code.cg2js/cg-dropping-outline.fasl")
(#x66b9790f8d09db48841342cf0676413c . "./code.cg2js/cg-font-dialog.fasl")
(#xdee98068c4f5f54310624610860d6b1f . "./code.cg2js/cg-icon-pixmap.fasl")
(#xdbb5b683583d2408478898464a36a6b9 . "./code.cg2js/cg-yes-no-list.fasl")
(#x5a566af760b2250c2610935731addf21 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xecaf975f82357adf95d4f89ca011e88d . "./code.cg2js/cg-lamp.fasl")
(#x9bfc97a8ee39ccf7675340bcc53ceee7 . "./code.cg2js/cg-drawable.fasl")
(#xb58b997368f669f0dbf49002efccd434 . "./code.cg2js/cg-choice-list.fasl")
(#xd10c6165b69d8fe832912a3362171e54 . "./code.cg2js/cg-curve.fasl")
(#xfec4923dc92514dbc69affd62ca0650f . "./code.cg2js/cg-base.fasl")
(#xfc5bf0beffdecd6ddd637b5f9d0a01ab . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x1bead55c60fd06f062ac79e933e0952c . "./code.cg2js/cg-printing.fasl")
(#x4071421bd9cf139ac551d85406274aa5 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x47373bd4be31e1171840586da9fda987 . "./code.cg2js/cg-hotspot.fasl")
(#x787ef1c23a2edc4e74603e6358518816 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xc1de085ebd4e930e4c634a76a6393eb3 . "./code.cg2js/cg-utility-dialog.fasl")
(#xee74814a95c51145d13a6c64cf4ef338 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xccfac06ffa0ec3aeece0793634918a18 . "./code.cg2js/cg-comtab.fasl")
(#xd7abeb0d864c3879f2a5f19bceba312a . "./code.cg2js/cg-os-widget.fasl")
(#xc7c3adf584da63acffba8e8a91c0388f . "./code.cg2js/cg-icon.fasl")
(#xbae3d16e17b85c0b378b1c1e5b05bab3 . "./code.cg2js/cg-dev.fasl")
(#xfb3ff589107d4bcca9f8288efb26d3d2 . "./code.cg2js/cg-calendar.fasl")
(#x9f16eb1fa5aba348d2c46200ba8998f4 . "./code.cg2js/cg-group-box.fasl")
(#x6b22a5e947848c2f407bc679a610b7f3 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x7bc08d45ed5f91f8f984a5e80f460d3b . "./code.cg2js/cg-bitmap-pane.fasl")
(#xa3c8fac71d53e24ab7383443af64d906 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xcda7697ee3c9c2f1894be8497f4792ad . "./code.cg2js/cg-nodes.fasl")
(#x4c10263944360b2980b1c24a73ee8b6b . "./code.cg2js/cg-intersect.fasl")
(#x63dfe2842373a82cac13f7b5b84fc4a2 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x76de7cb629c30a61afc61bed6c5cad9a . "./code.cg2js/cg-class-support.fasl")
(#x1752888011da971d1f51375d060c1b1b . "./code.cg2js/cg-dialog-item.fasl")
(#x690f2a03e5b2ef44049e526bff65b087 . "./code.cg2js/cg-plot-widget.fasl")
(#x477634c913e756c79d5b7cba56380614 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xa01009f5b0999d5297e4ad438b8a1015 . "./code.cg2js/cg-up-down-control.fasl")
(#x3975860650865b4d3bbc9a59797f08f0 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x929edc6dd9c2fe168d14f23b91957267 . "./code.cg2js/cg-status-bar.fasl")
(#x2d85baa52cb9bf2c9e6e00ee7ace6137 . "./code.cg2js/cg-class-grid.fasl")
(#xd322ae6451e2388b40cc830d19a9a5a8 . "./code.cg2js/cg-wrap-string.fasl")
(#x395e582a9505383a31b455c017116a94 . "./code.cg2js/cg-layout.fasl")
(#xea94b4febf21aef66a886df99c1848ce . "./code.cg2js/cg-lisp-widget.fasl")
(#x144510b07b15cff1b28654844fe6f06d . "./code.cg2js/cg-button.fasl")
(#x9e2d1c7e90513011bd61c90cebf76e5a . "./code.cg2js/cg-string-dialog.fasl")
(#x70cf541f78601381a6b4dee96a419493 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x3e62bc7c689b6222d5678d895b601fce . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xbccb704ddf12672fe561a377c7234f4e . "./code.cg2js/cg-bitmap-stream.fasl")
(#xbdfc7455f27c78199ff597fd7fed11ff . "./code.cg2js/cg-static-text.fasl")
(#x473183a2da37ad6570f30c566c254e2b . "./code.cg2js/cg-outline.fasl")
(#xa95902a1c5cc2e0af9e7f32f1361bd07 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x20fb78c40ab8143c4636542f86661725 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xb75a06e5b8e9ecf7dcd8e515230da7a9 . "./code.cg2js/cg-picture-widget.fasl")
(#x844eed440638d536bc30edd04f1c12c1 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x7b23e7749ada060fc16785d4eab02c58 . "./code.cg2js/cg-chart-widget.fasl")
(#x0871ecf91d8668cfed6f3adccfaa1251 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x5505c298ba4353cf586c276cc19d9d9b . "./code.cg2js/cg-lisp-text.fasl")
(#xd002c55708fd65b61b4d0c9dee1ee0a3 . "./code.cg2js/cg-palette.fasl")
(#xc263b3576722b79b8c7ca097f0a9eb1f . "./code.cg2js/cg-property.fasl")
(#x36069d0dae3bc7a2375ee1be4cc9c52c . "./code.cg2js/cg-intersect-posbox.fasl")
(#x9ef3024b51aa650817bde548eac830fb . "./code.cg2js/cg-file-dialog.fasl")
(#xd6183641aaf21a96ae0c1c51a6cbc930 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x283279d1b074df5c31e14e2d93b46c16 . "./code.cg2js/cg-get-pixmap.fasl")
(#x8ca10cfa7cd0023569cc88ec0c9de686 . "./code.cg2js/cg-tray.fasl")
(#xec66b675f45c70a9f2adf5963810c14d . "./code.cg2js/cg-acache.fasl")
(#xa6a020c4fe10321b8fdc255305062cc1 . "./code.cg2js/cg-message-dialog.fasl")
(#x91933a61548f228f8bb872cae1e269f4 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xb5721eac23832de2e6f326666631f3a8 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x413ba7f876bb97929f9ce87717d2a02e . "./code.cg2js/cg-tab-control.fasl")
(#x7490be2c4b908769fa27ca2495c625c9 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x30bc4dc5c00b397f0643596d1d8c1514 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xb9e8bf6c06c425b4e8b8cea3c8121384 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xe0c93abd8c8aa0eeb84d988b9879d3ca . "./code.cg2js/cg-menu.fasl")
(#xe613e8e83664c921f7ca813350ecdc1a . "./code.cg2js/cg-get-position.fasl")
(#xb206d182fa447b5a40ef9226df5e6e7f . "./code.cg2js/cg-trackbar.fasl")
(#xf6edc674570f8380b2839475545f065e . "./code.cg2js/cg-selected-object.fasl")
(#x369ce8d3aeeffe0cb2cb11ecbfe68081 . "./code.cg2js/cg-list-view.fasl")
(#xeba45edb69ade569bf837a8418c41c82 . "./code.cg2js/cg-html-widget.fasl")
(#x641657fdf734782262c88120ea3a53da . "./code.cg2js/cg-sample-file-menu.fasl")
(#x7b58c7f02e7b48616fb70a49fde830dd . "./code.cg2js/cg-text-widget.fasl")
(#xa7f2a0a9cf3a0b911384fc89d574cc8c . "./code.idejs/ide-base.fasl")
(#xce700d551ef8036bfbd8b4e3f5cc12ec . "./code.idejs/ide-process-browser.fasl")
(#xd0c5efe1a957964e5417449b5d230629 . "./code.idejs/ide-patch-dialog.fasl")
(#x12bbeffca55e872b998e7a2d1c67b0f2 . "./code.idejs/ide-keyshort.fasl")
(#x75b076906b0f798607cb25c452c5ecfe . "./code.idejs/ide-window-list.fasl")
(#xf64c0c7caa7770e3b624421757da0ce0 . "./code.idejs/ide.fasl")
(#xcd5e2ab770efab1e1f979870a3d76ef9 . "./code.idejs/ide-editor.fasl")
(#x781befd672431a80fb79fd69dff9fe6e . "./code.idejs/ide-clipboard.fasl")
(#xef2104861088983e4a96b2b16422f052 . "./code.idejs/ide-trace.fasl")
(#xe7d028812958114c2990e59bee429e47 . "./code.idejs/ide-class-browser.fasl")
(#xffccf7ccd102914026c97e8829427ff3 . "./code.idejs/ide-grapher.fasl")
(#xef7f5bd4c503b01288a99df6cedbf1c0 . "./code.idejs/ide-apropos.fasl")
(#xa384a23b25a03a4850eb2fd80b383ea5 . "./code.idejs/ide-navigator.fasl")
(#x6205d2eb6ef05321cb9b841b8fdebe97 . "./code.idejs/ide-find-definitions.fasl")
(#x0894d4e517dd9ab987c9d3be27031758 . "./code.idejs/ide-project.fasl")
(#xd2536cd78e4313ae958a1cc95051b247 . "./code.idejs/ide-class-grapher.fasl")
(#x880deead809492eb265d97fc74ba07e0 . "./code.idejs/ide-acache.fasl")
(#xe641228629c68b3c661a42dda642dec4 . "./code.idejs/ide-find-in-files.fasl")
(#x8517181026d661e21d22e44d248d0560 . "./code.idejs/ide-inspector.fasl")
(#x209ce2d6c6d37f3ae82ef848c41a9e1b . "./code.idejs/ide-intro.fasl")
(#x0b3d0c5d4449ec59ff5ff70af81bc0b4 . "./code.idejs/ide-package-list.fasl")
(#x0ee3d2464f8a49b269449ef6bd54d0c9 . "./code.idejs/ide-dev.fasl")
(#x2914a61638acb2c2838394daf09749a0 . "./code.idejs/ide-form.fasl")
(#x29c4a4cb691acde2a2fe4116aa720ac8 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
